;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Phanto, adapted by yoshicookiezeus
;;
;; For usage with Tessera
;; Number of extra bytes used: 2
;; Extra byte 1: graphics tile used by the sprite
;; Extra byte 2: hfxxpppt
;;		h - if set, the sprite will only hurt Mario if he is carrying a key
;;		f - if set, the sprite will flip horizontally to always face Mario
;;		x - unused
;;		p - sprite palette used for the graphics tile
;;		t - graphics page used for the sprite graphics
;; Requires the shared subroutine patch to be applied to your ROM
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

incsrc subroutinedefs.asm

			!RAM_SpriteSpeedY	= $AA
			!RAM_SpriteSpeedX	= $B6
			!RAM_SpriteState	= $C2

			!RAM_ExtraByte1		= $7FAB40
			!RAM_ExtraByte2		= $7FAB4C
			!RAM_ExtraByte3		= $7FAB58
			!RAM_ExtraByte4		= $7FAB64
			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Main:			PHB
			PHK
			PLB
			JSR SpriteMain
			PLB
Init:			RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
AccelerationX:		db $02,$FE
MaxSpeedX:		db $28,$D8
AccelerationY:		db $02,$FE
MaxSpeedY:		db $18,$E8
			
SpriteMain:		JSR SpriteGraphics
			LDA $14C8,x		;\ if sprite state not normal
			CMP #$08		; |
			BNE Return		;/ return
			LDA $9D			;\ if sprites locked,
			BNE Return		;/ return

			JSR SubHasKey		;\ if Mario has a key,
			CPY #$01		; |
			BEQ Chase		;/ go to main movement routine

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Secondary movement routine
;
; This routine is executed if Mario isn't holding a key, and will slow Phanto down
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

			LDA !RAM_ExtraByte2,x
			BMI DontHurtMario

			JSL $01A7DC		; interact with Mario

DontHurtMario:		LDA $14			;\ only change speeds every second frame
			AND #$01		; |
			BNE ApplySpeed		;/

			LDA !RAM_SpriteSpeedX,x	;\ if x speed zero, 
			BEQ XSpeedZero		;/ branch
			BPL XSpeedPositive	; if positive, branch
			INC A
			INC A
XSpeedPositive:		DEC A
			STA !RAM_SpriteSpeedX,x

XSpeedZero:		LDA !RAM_SpriteSpeedY,x	;\ if y speed zero,
			BEQ YSpeedZero		;/ branch
			BPL YSpeedPositive	; if positive, branch
			INC A
			INC A
YSpeedPositive:		DEC A
			STA !RAM_SpriteSpeedY,x
YSpeedZero:		BRA ApplySpeed

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Main movement routine
;
; If Mario is holding a key, this routine moves Phanto towards him, and accelerates it
; if the speed cap hasn't been hit
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

Chase:			JSL $01A7DC		; interact with Mario

			LDA $14			;\ only change speeds every fourth frame
			AND #$03		; |
			BNE ApplySpeed		;/

			JSL !SubHorizPos	;\ if max horizontal speed in the appropriate
			LDA !RAM_SpriteSpeedX,x	; | direction achieved,
			CMP MaxSpeedX,y		; |
			BEQ MaxXSpeedReached	;/ don't change horizontal speed
			CLC			;\ else,
			ADC AccelerationX,y	; | accelerate in appropriate direction
			STA !RAM_SpriteSpeedX,x	;/

MaxXSpeedReached:	JSL !SubVertPos2	;\ if max vertical speed in the appropriate
			LDA !RAM_SpriteSpeedY,x	; | direction achieved,
			CMP MaxSpeedY,y		; |
			BEQ ApplySpeed		;/ don't change vertical speed
			CLC			;\ else,
			ADC AccelerationY,y	; | accelerate in appropriate direction
			STA !RAM_SpriteSpeedY,x	;/

ApplySpeed:		JSL $018022		; apply x speed
			JSL $01801A		; apply y speed

Return:			RTS			         

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HAS_KEY
;
; If Mario is holding a key, the Y register is set to 1 after this routine has been run
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;                    

SubHasKey:		PHX			; push Phanto sprite number
			LDY #$00		; clear Y register
			LDA $1470		;\ if carriying flag not set,
			BEQ .Return		;/ no need to check for key
                    
                    
			LDX #$0A		; setup loop variable
.LoopStart		LDA $9E,x		;\ if sprite currently being checked isn't key,
			CMP #$80		; | (key = sprite # 80)
			BNE .NextX		;/ no need to check anything else for that sprite
			LDA $14C8,x		;\ if sprite is key, check if it is being carried
			CMP #$0B		; |
			BNE .NextX		;/ if it isn't, no more checks
			BRA .HasKey		; if it is, get out of loop
.NextX			DEX			; increase number of sprite to check
			BPL .LoopStart		; if still sprites left to check, repeat loop
			BRA .Return		; if not, return
                    
.HasKey			INY			; increase Y register to show that key is being carried                    
.Return			PLX			; get Phanto sprite number back
			RTS			; and return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GENERIC GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpriteGraphics:		JSL !SubHorizPos
			TYA
			STA $02

			JSL !GetDrawInfo

			LDA !RAM_ExtraByte2,x
			AND #$40
			STA $03

			LDA $00			; set x position of the tile
			STA $0300,y

			LDA $01			; set y position of the tile
			STA $0301,y

			LDA !RAM_ExtraByte1,x
			STA $0302,y

			LDA !RAM_ExtraByte2,x
			AND #$0F
			PHX
			LDX $02
			BNE DontFlipX
			ORA $03
DontFlipX:		ORA $64			; add in the priority bits from the level settings
			STA $0303,y		; set properties
			PLX

			LDY #$02		; the tiles drawn were 16x16
			LDA #$00		; one tile was drawn
			JSL $01B7B3		; finish OAM write
			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dl Init,Main